unit UExtImgLst;
//*****************************************************************************
// Unit UExtImgLst      - Diese Unit enthlt die Klasse TExtImgLst, sie
// ---------------        verwaltet die verschiedenen Spiel-Bilder (die auch
//                        unterschiedlich gro sein knnen) in einer Liste.
//                        Ein Element der Liste besteht jeweils aus dem Bild
//                        und der Gre (in %) des Bildes auf dem Spielfeld
//
// ffentliche-Methoden:
// ---------------------
// Init                 - Initialsiert die Bilder-Liste
// Count                - Gibt die Anzahl der Bilder in der Liste zurck
// AddBmp               - Fgt ein neues Bild ans Ende der Bilderliste
// ChangeBmp            - ndert Bitmap / Gre eines Bildes in der Bilderliste
// GetBmp               - Ermittelt Bitmap und die dazugehrige Gre (in %)
//                        eines Bildes aus der Bilderliste
// DelBmp               - Entfernt ein Element aus der Liste
// AllFree              - Entfernt alle Elemente aus der Liste
//
// Verwendet eigene Units:
// -----------------------
// UType                - enthlt wichtige Typen und Konstanten wie z.B.
//                        THighscore, TPlayerName etc.
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface

uses Classes, Graphics, UType;

type
 TExtImgLst    = class
                        private
                        ListPtr  : TBmpLstPtr;
                        LastPtr  : TBmpLstPtr;
                        ImgCount : Word;
                        public
                        procedure Init;
                        function  Count : Word;
                        procedure AddBmp(Bmp : TBitmap; Size : Byte);
                        procedure ChangeBmp(Num : Word; Bmp : TBitmap; Size : Byte);
                        function  GetBmp(Num : Word; var Size : Byte) : TBitmap;
                        procedure DelBmp(Num : Word);
                        procedure AllFree;
                 end;

implementation

//*****************************************************************************
// Initialisiert die Bilderliste
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
procedure TExtImgLst.Init;
begin
 ListPtr  := nil;
 LastPtr  := nil;
 ImgCount := 0;
end;


//*****************************************************************************
// < Gibt die Anzahl der Bilder in der Liste zurck
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
function TExtImgLst.Count : Word;
begin
 Count := ImgCount;
end;


//*****************************************************************************
// Fgt ein neues Bild ans Ende der Bilderliste
//
//  ->   Bmp         Bitmap, das an die Liste angefgt werden soll
//  ->   Size        Gre des angefgten Bildes (in %) auf dem Spielfeld
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
procedure TExtImgLst.AddBmp(Bmp : TBitmap; Size : Byte);
var
 tmp : TBmpLstPtr;
begin
 // bereite neues Bitmap-Element vor
 new(tmp);
 // Inc(LstDebug);                        // zum Debuggen der Zeigerfunktionen
 tmp^.Size                 := Size;
 tmp^.Bmp                  := TBitmap.Create;
 tmp^.Bmp.Assign(Bmp);
 tmp^.Bmp.Transparent      := C_TRANSPARENT;
 tmp^.Bmp.TransparentColor := clBlack;
 tmp^.Next                 := nil;
 tmp^.Prev                 := nil;

 if ListPtr = nil then        // Liste leer?
    ListPtr := tmp            // neues Element am Anfang
 else
 begin
    tmp^.Prev := LastPtr;     // altes letztes El. ist Vorgnger
    LastPtr^.Next := tmp;     // altes letztes El. zeigt auf neues letztes El.
 end;

 LastPtr := tmp;              // neues El. ist letztes El.
 Inc(ImgCount);
end;


//*****************************************************************************
// ndert Bitmap bzw. Gre (in %) eines Bildes in der Bilderliste
//
//  ->   Num         Nummer des zu ndernden Elements in der Liste
//                   (erste Element ist Eins!)
//  ->   Bmp         Bitmap des neuen Bildes
//  ->   Size        Gre des Bildes (in %) auf dem Spielfeld
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
procedure TExtImgLst.ChangeBmp(Num : Word; Bmp : TBitmap; Size : Byte);
var
 tmp : TBmpLstPtr;
begin
 if Num <= ImgCount then         // gibt es berhaupt so viele Bilder?
 begin
    tmp := ListPtr;              // erstes Element
    while num > 1 do             // durchlaufe Liste bis zum richtigen Element
    begin
       tmp := tmp^.Next;
       Dec(Num);
    end;
    tmp^.Bmp.Assign(Bmp);                    // weise neues Bild zu
    tmp^.Bmp.Transparent := C_TRANSPARENT;   // setze Transparenz
    tmp^.Bmp.TransparentColor := clBlack;    // und Transparenzfarbe
    tmp^.Size := Size;                       // setze Gre (in %)
 end;
end;


//*****************************************************************************
// Ermittelt Bitmap und die dazugehrige Gre (in %) eines Bildes aus der
// Liste
//
//  ->   Num         Nummer des zu geforderten Elements in der Liste
//                   (erste Element ist Eins!)
//  <-   Size        Variable, die Gre (in %) aufnimmt
// <                 Gibt das Bitmap des angegebenen Bildes zurck
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
function TExtImgLst.GetBmp(Num : Word; var Size : Byte) : TBitmap;
var
 tmp : TBmpLstPtr;
begin
 if Num <= Count then         // gibt es berhaupt so viele Bilder?
 begin
    tmp := ListPtr;           // erstes Element
    while num > 1 do          // durchlaufe Liste bis zum richtigen Element
    begin
       tmp := tmp^.Next;
       Dec(Num);
    end;
    Size   := tmp^.Size;      // Speicher Gre (in %) fr Rckgabe
    GetBmp := tmp^.Bmp;       // Gebe Bitmap zurck
 end;
end;


//*****************************************************************************
// Entfernt ein Element aus der Liste und gibt belegten Speicher wieder frei
//
//  ->   Num         Nummer des zu geforderten Elements in der Liste
//                   (erste Element ist Eins!)
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
procedure TExtImgLst.DelBmp(Num : Word);
var
 tmp, tmpPrev, tmpPost : TBmpLstPtr;
begin
 if Num <= Count then               // gibt es berhaupt so viele Bilder?
 begin
    tmp := ListPtr;                 // erstes Element
    while num > 1 do                // durchlaufe Liste bis zum richtigen Element
    begin
       tmp := tmp^.Next;
       Dec(Num);
    end;
    tmpPrev := tmp^.Prev;           // merke Vorgnger Element
    tmpPost := tmp^.Next;           // merke Nachfolger Element

    if tmpPrev <> nil then          // wenn Vorgnger vorhanden,
       tmpPrev^.Next := tmpPost;    // Vorgnger soll auf Nachfolger zeigen

    if tmpPost <> nil then          // wenn Nachfolger vorhanden,
       tmpPost^.Prev := tmpPrev;    // Nachfolger soll auf Vorgnger zeigen

    if tmp = ListPtr then           // Falls erstes Element,
       ListPtr := tmp^.Next;        // Nachfolger wird neues erstes Element

    if tmp = LastPtr then           // Falls letztes Element,
       LastPtr := tmp^.Prev;        // Vorgnger wird neues letztes Element

    tmp^.Bmp.Free;                  // Gebe Bitmap des zu lschenden El. frei
    Dispose(tmp);                   // Gebe Speicher des zu lschenden El. frei
    // Dec(LstDebug);                  // zum Debuggen der Zeigerfunktionen
    Dec(ImgCount);                  // Korrigiere die Anzahl der Bilder in der Liste
 end;
end;


//*****************************************************************************
// Leert die gesamte Bilderliste indem alle Elemente entfernt werden
// Der gesamte belegte Speicher wird wieder freigegeben
//
// Achtung:          Es wird nur die Liste aufgelst, dass Klassenobjekt
//                   muss spter mit Free freigegeben werden!
//
// as, 25.09.2002 - Erste Version
//*****************************************************************************
procedure TExtImgLst.AllFree;
var
 tmp : TBmpLstPtr;
begin
 tmp := ListPtr;                    // erstes Element
 while tmp <> nil do                // durchlaufe Liste bis zum Ende
 begin
    ListPtr := tmp;                 // merke aktuelle Position
    tmp := tmp^.Next;               // gehe ein Element weiter
    if ListPtr^.Bmp <> nil then     // Wenn Bitmap-vorhanden,
       ListPtr^.Bmp.Free;           // gebe es frei
    Dispose(ListPtr);               // und gebe aktuelles Element frei
    // Dec(LstDebug);                  // Zum Debuggen der Zeigerfunktionen
 end;
 ListPtr := nil;
 LastPtr := nil;
end;

end.
