unit UObjEnemy;
//*****************************************************************************
// Unit UObjEnemy       - Diese Unit enthlt die Klasse TEnemy inklusive
// --------------         aller Deklarationen und Member-Methoden
//                      - Die Klasse TEnemy dient zur Anzeige und Bewegung
//                        der Gegner-Raumschiffe
//                      - auerdem steuert sie die Gegner-Schsse/Kollisionen
//
// ffentliche-Methoden:
// ---------------------
// Init                 - Initialisiert das Gegner-Objekt, die Gegner-Liste und
//                        Schu-Liste
// AddEnemies           - Fgt zu einem Level alle Gegner in das Spielfeld ein
// Draw                 - Bewegt udn zeichnet alle Gegner auf dem Spielfeld
// DrawEnemyShoot       - Bewegt und zeichnet alle Schsse der Gegner
// HitEnemy             - berprft, ob Gegner von dem Spieler getroffen ist
// HitEnemyShoot        - berprft, ob Gegner-Schu den Spieler getroffen hat
// NoEnemy              - Gibt zurck, ob noch Gegner auf dem Spielfeld sind
// ShootUpdate          - Initialisiert alle Gegner mit neuen Schssen
// AddEnemyShoot        - Fgt einen Gegner-Schu an einer bestimmten Position
//                        im Spielfeld ein
// Resize               - Korrigiert die Bildgren, Positionen und Geschw. der
//                        Gegner und der Gegner-Schsse
// SetEnemySpeed        - Setzt die Geschwindigkeit der Gegner neu
// SetBitmapSize        - ndert die Gren der Gegner-Bilder passend zur
//                        neuen Spielfeldgre
// FreeBmp              - Gibt die Bilderlisten fr die Gegner wieder frei
// FreeLst              - Entfernt alle Gegner und Gegner-Schsse vom Spielfeld
//
// Verwendet eigene Units:
// -----------------------
// UType                - Enthlt wichtige Typen
// UFile                - enthlt Routinen zum Lesen der Spiele-Grafiken aus
//                        verschiedenen Dateien
// UObjFighter          - Enthlt die Elternklasse "TFighterObject", von der
//                        sich diese Klasse ableitet
// UObjEnemy            - Enthlt Routinen fr das Gegnerverhalten, insbesondere
//                        das Schussverhalten und Trefferverhalten
// UObjText             - zeigt Infos auf dem Spielfeld an
// UObjFire             - zeichnet Explosionen auf das Spielfeld
// UExtImgLst           - Stellt Routinen fr Bilderlisten zur Verfgung
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface
uses Types, SysUtils, Graphics, Math, MMSystem, UType, UObjFighter, UObjFire, UExtImgLst;

type
 TEnemy  = class(TFighterObject)
            private
            EnemyLst, EnemyShootLst       : TFighterPtr;
            EnemyBmp                      : TExtImgLst;
            EnemyBmpOrig                  : TExtImgLst;
            EnSpeed                       : Real;
            EnSpeedOrig                   : Real;
            procedure AddEnemyCount(Max_x, Max_y : integer; FireCount : Byte; Level : Word; Count : Word);
            procedure NewRandomPos(Playfield : TRect; var Enemy : TFighterPtr);
            procedure EnemyMove(Playfield : TRect; var Enemy : TFighterPtr);
            function  InitEnemyShoot(Count : Byte) : TEnemyShootSet;

            public
            procedure Init(PlayField : Integer; GameBitmaps : TExtImgLst);
            procedure AddEnemies(Max_x, Max_y : integer; FireCount : Byte; Level : Word; StartCount, IncCount : Word);
            procedure Draw(Canvas : TCanvas; TimerValue : TEnemyShoot);
            procedure DrawEnemyShoot(Canvas : TCanvas);
            function  HitEnemy(Rect : TRect; Fire : TFire) : Boolean;
            function  HitEnemyShoot(rect : TRect) : boolean;
            function  NoEnemy : Boolean;
            procedure ShootUpdate(Count : Byte);
            procedure AddEnemyShoot(x,y : Word; Max_X, Max_Y : Integer);
            procedure Resize(OldField, NewField : Integer);
            procedure SetEnemySpeed(Speed : Real; PlayField : Integer);
            procedure SetBitmapSize(NewField : Integer);
            procedure FreeBmp;
            procedure FreeLst;
          end;

implementation

uses UFile;

//*****************************************************************************
// Initialisiert das Gegner-Objekt, die Gegner-Liste und Schu-Liste
//
//  ->   PlayField   Spielfeldgre
//  ->   GameBitmaps Liste mit den Spiele-Grafiken
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.Init(PlayField : Integer; GameBitmaps : TExtImgLst);
var
 i : Word;
 Percent : Byte;
 tmpBmp : TBitmap;
begin
 EnemyLst      := nil;                                // Gegnerliste leer
 EnemyShootLst := nil;                                // Schuss-Liste leer
 EnemyBmp      := TExtImgLst.Create;                  // erzeuge Liste fr Gegner-Grafiken
 EnemyBmpOrig  := TExtImgLst.Create;                  // Kopie der Liste mit den Bilder in Original-Gre
 tmpBmp        := TBitmap.Create;                     // temp. Bild erzeugen

 for i:=3 to GameBitmaps.Count do                     // fange bei 3 an, da Bild 1+2 sind Player-Grafiken
 begin
    tmpBmp.Assign( GameBitmaps.GetBmp(i, Percent) );  // lese jeweils ein Bild aus der Liste
    EnemyBmp.AddBmp(tmpBmp, Percent);                 // und fge es in die Gegner-Bilderliste
    EnemyBmpOrig.AddBmp(tmpBmp, Percent);             // und die Gegner-Bilderliste fr Original-Gre
 end;

 tmpBmp.Free;                                         // gebe temp. Bild wieder frei!
end;


//*****************************************************************************
// Fgt beliebige Anzahl Gegner eines Levels in das Spiel ein
//
//  ->   Max_x       Spielfeldbreite
//  ->   Max_y       Spielfeldhhe
//  ->   FireCount   Wieviele Schsse sollen die Gegner pro Minute abfeuern
//  ->   Level       aus welchem Level sollen die Bilder genommen werden
//  ->   Count       Wieviele Gegner sollen hinzugefgt werden
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.AddEnemyCount(Max_x, Max_y : integer; FireCount : Byte; Level : Word; Count : Word);
var
 Data    : TFighterRec;
 i       : byte;
 Percent : Byte;
 tmpBmp  : TBitmap;
begin
 tmpBmp := TBitmap.Create;                               // temp. Bild erzeugen
 if EnemyBmp.Count < Level+1 then                        // Wenn kein Bild fr das Level mehr in Liste
    Level := EnemyBmp.Count-1;                           // benutze letztes Bild aus Liste
 tmpBmp.Assign( EnemyBmp.GetBmp(Level+1, Percent) );     // weise das entspr. Gegner Bild dem temp. Bild zu

 for i:=1 to Count do                                    // fr jeden einzufgenden Gegner durchlaufen
 begin
    Data.Lives := Level;                                 // merke Level
    Data.Width := tmpBmp.Width;                          // setze Breite
    Data.Height:= tmpBmp.Height;                         // setze Hhe
    with Data.Move do
    begin
       Pos.X      := Random(Max_X - Data.Width -2)+1;    // zufllige Position des Gegners im Spielfeld
       Pos.Y      := Random(Max_Y - Data.Height -2)+1;
       NewPos.X   := Pos.X;                              // noch kein Ziel fr Bewegung setzen
       NewPos.Y   := Pos.Y;
       Shoot      := InitEnemyShoot(FireCount);          // Mache Gegner schubereit
    end;
    Add(EnemyLst, Data);                                 // Fge Gegner hinzu
 end;

 tmpBmp.Free;
end;


//*****************************************************************************
// Fgt alle Gegner in das Spielfeld ein, die in einem bestimmten Level
// vorhanden sein mssen
//
//  ->   Max_x       Spielfeldbreite
//  ->   Max_y       Spielfeldhhe
//  ->   FireCount   Schussfrequenz der Gegner
//  ->   Level       welches Level ist gemeint
//  ->   StartCount  Anzahl der Gegner in Level 1
//  ->   IncCount    Erhhung der Anzahl pro Level
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.AddEnemies(Max_x, Max_y : Integer; FireCount : Byte; Level : Word; StartCount, IncCount : Word);
var
 i      : Word;
begin
 AddEnemyCount(Max_x, Max_y, FireCount, 1, StartCount);     // fge Gegner fr Level 1 ein
 for i:=2 to Level do
    AddEnemyCount(Max_x, Max_y, FireCount, i, IncCount);    // fge Gegner fr jedes weitere Level ein
end;


//*****************************************************************************
// Ermittelt eine neue Zufalls-Position zu der ein Gegner sich bewegen soll
//
//  ->   Playfield   Rechteck des Spielfelds
// <->   Enemy       Zeiger auf einen Gegner, der ein neues Zeil bentigt
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.NewRandomPos(Playfield : TRect; var Enemy : TFighterPtr);
begin
 with Enemy^.Data.Move do
 begin
    NewPos.X   := Random(Playfield.Right  - 2 - Enemy^.Data.Width ) + 1;   // berechne zufllige X-Position
    NewPos.Y   := Random(Playfield.Bottom - 2 - Enemy^.Data.Height) + 1;   // berechne zufllige Y-Position
 end;
end;


//*****************************************************************************
// Bewegt ein Gegner auf dem Spielfeld von der aktuellen Position zum
// nchsten Zwischenziel
//
//  ->   Playfield   Rechteck des Spielfelds
// <->   Enemy       Zeiger auf Gegner, der sich bewegen soll
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.EnemyMove(Playfield : TRect; var Enemy : TFighterPtr);
var
 lenX, lenY, lenG : Real;
 dx, dy : Real;
begin
 with Enemy^.Data.Move do
 begin
    lenX := NewPos.X - Pos.X;                   // berechne X-Abstand bis zum Ziel
    lenY := NewPos.Y - Pos.Y;                   // berechne Y-Abstand bis zum Ziel
    lenG := Sqrt( Sqr(lenX) + Sqr(lenY) );      // berechne direkten Abstand zum Ziel

    if LenG <> 0 then                           // wenn noch nicht am Ziel angelangt
    begin
       dx   := (lenX / lenG) * EnSpeed;         // berechne X-Annherung
       dy   := (lenY / lenG) * EnSpeed;         // berechne Y-Annherung
       Pos.X := Pos.X + dx;                     // bewege Gegner in X-Richtung
       Pos.Y := Pos.Y + dy;                     // bewege Gegner in Y-Richtung
       if LenG < EnSpeed then                   // Wenn quasi am Ziel angelangt
          NewRandomPos(Playfield, Enemy);       // berechne ein neues Ziel
    end
    else
       NewRandomPos(Playfield, Enemy);
 end;
end;


//*****************************************************************************
// Bewegt und zeichnet alle Gegner der Liste auf dem Spielfeld 
//
//  ->   Canvas      Zeichenflche, auf der gezeichnet wird
//  ->   TimerValue  aktuellen Gegner-Timerstand -> fr Gegner-Schsse
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.Draw(Canvas : TCanvas; TimerValue : TEnemyShoot);
var
   tmpPtr   : TFighterPtr;
   Percent  : Byte;
begin
 tmpPtr := EnemyLst;

 while tmpPtr <> nil do                               // durchlaufe jeden Gegner der Liste
 begin
    EnemyMove(Canvas.ClipRect, tmpPtr);               // bewege Gegner
    with tmpPtr^.Data do
    begin
       if TimerValue in Move.Shoot then               // ist es Zeit fr den Gegner einen Schu abzusetzen?
       begin
          AddEnemyShoot(Round(Move.Pos.X), Round(Move.Pos.Y), Canvas.ClipRect.Right, Canvas.ClipRect.Bottom);
          Move.Shoot := Move.Shoot - [TimerValue];    // nach abgefeuertem Schu wird dieser aus Schumenge entfernt
       end;
       Canvas.Draw(Round(Move.Pos.X), Round(Move.Pos.Y), EnemyBmp.GetBmp(Lives+1, Percent));
    end;
    tmpPtr := tmpPtr^.next;                           // gehe zum nchsten Element
 end;
end;


//*****************************************************************************
// Bewegt und zeichnet alle Schsse der Gegner
//
//  ->   Canvas      Zeichenflche, auf der gezeichnet wird
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.DrawEnemyShoot(Canvas : TCanvas);
var
   tmpPtr, delPtr   : TFighterPtr;
   Percent          : Byte;
begin
 tmpPtr := EnemyShootLst;

 while tmpPtr <> nil do                                     // durchlaufe alle Schsse
 begin
    with tmpPtr^.Data do
    begin
       delPtr := tmpPtr;                                    // merke aktuellen Schu-Zeiger
       Inc(y, Speed);                                       // bewege Schu
       Canvas.Draw(x, y,  EnemyBmp.GetBmp(1, Percent) );    // zeichne Schu

       tmpPtr := tmpPtr^.next;                              // merke nchsten Schu vor
       if y > Canvas.ClipRect.Bottom then                   // wenn Schu schon aus Spielfeld heraus
          FreePtr(EnemyShootLst, delPtr);                   // gebe ihn wieder frei
    end;
 end;
end;


//*****************************************************************************
// berprft, ob ein Gegner von dem Spieler getroffen ist
//
//  ->   Rect        Rechteck des Spieler-Schusses
//  ->   Fire        Objekt zum darstellen der Explosionen
// <                 Gibt zurck, ob ein Gegner getroffen wurde
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
function TEnemy.HitEnemy(Rect : TRect; Fire : TFire) : Boolean;
var
 tmpPtr   : TFighterPtr;
 delPtr   : TFighterPtr;
 Rec1     : TRect;
begin
 tmpPtr     := EnemyLst;                                       // gehe zum Anfang der Gegner-Liste
 HitEnemy   := False;

 while tmpPtr <> nil do                                        // durchlaufe alle Gegner der Liste
 begin
    delPtr := tmpPtr;
    with tmpPtr^.Data do
    begin                                                      // berechne Rechteck des Gegners
       Rec1.Left   := Round(Move.Pos.X);
       Rec1.Top    := Round(Move.Pos.Y);
       Rec1.Right  := Rec1.Left + Width;
       Rec1.Bottom := Rec1.Top  + Height;

       if IsCollision(Rec1, Rect) then                         // Kollsion zwischen Gegner- und Schurechteck
       begin
          tmpPtr   := tmpPtr^.next;                            // merke nchsten Gegner
          HitEnemy := True;                                    // Gegner wurde getroffen
          Fire.AddFire(Rec1.Left,Rec1.Top,Rec1.Right,Rec1.Bottom,C_FIRE_SPEED);     // Explsoion anzeigem
          FreePtr(EnemyLst, delPtr);                           // Gegner-Objekt entfernen
       end
       else
          tmpPtr := tmpPtr^.next;
    end;
 end;
end;


//*****************************************************************************
// berprft, ob ein Gegner-Schu den Spieler getroffen hat
//
//  ->   Rect        Rechteck des Spielers
// <                 Gint zurck, ob der Spieler getroffen wurde
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
function  TEnemy.HitEnemyShoot(Rect : TRect) : boolean;
var
   tmpPtr   : TFighterPtr;
   delPtr   : TFighterPtr;
   Rec1     : TRect;
begin
 tmpPtr        := EnemyShootLst;             // gehe auf Anfang der Schu-Liste
 HitEnemyShoot := False;                     // noch kein Treffer

 while tmpPtr <> nil do                      // durchlaufe alle Schsse der Liste
 begin
    delPtr := tmpPtr;                        // merke aktuellen Zeiger
    with tmpPtr^.Data do
    begin
       Rec1.Left   := X;                     // berechne Schu-Rechteck
       Rec1.Top    := Y;
       Rec1.Right  := X + Width;
       Rec1.Bottom := Y + Height;
       if IsCollision(Rec1, Rect) then       // Wenn Schu- mit Spielerrechteck kollidiert
       begin
          tmpPtr := tmpPtr^.next;            // nchsten Schu vormerken
          HitEnemyShoot := true;             // Spieler wurde getroffen
          FreePtr(EnemyShootLst, delPtr);    // Der Schu kann freigegeben werden
       end
       else
          tmpPtr := tmpPtr^.next;
    end;
 end;
end;


//*****************************************************************************
// <                 Gibt zurck, ob noch Gegner auf dem Spielfeld sind
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
function TEnemy.NoEnemy : Boolean;
begin
 NoEnemy := EnemyLst = nil;   // wenn Liste nicht leer -> noch Gegner da
end;


//*****************************************************************************
// Initialisiert alle Gegner mit neuen Schssen
//
//  ->   Count       Wieviele Schsse sollen pro Minute abgefeuert werden?
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.ShootUpdate(Count : Byte);
var
 tmpPtr : TFighterPtr;
begin
 tmpPtr := EnemyLst;                                     // zeige auf Anfang der Liste
 while tmpPtr <> nil do                                  // durchlaufe alle Gegner
 begin
    tmpPtr^.Data.Move.Shoot := InitEnemyShoot(Count);    // setze neue Schu-Menge fr Gegner
    tmpPtr := tmpPtr^.next;                              // und ab zum nchsten Gegner
 end;
end;


//*****************************************************************************
// Fgt einen Gegner-Schu an einer bestimmten Position im Spielfeld ein
//
//  ->   x,y         Position des Schusses auf dem Spielfeld
//  ->   Max_X       Spielfeldbreite
//  ->   Max_Y       Spielfeldhhe
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.AddEnemyShoot(x,y : Word; Max_X, Max_Y : Integer);
var
 Data    : TFighterRec;
 Percent : Byte;
 tmpBmp  : TBitmap;
begin
 tmpBmp        := EnemyBmp.GetBmp(1, Percent);                    // ermittel Zeiger auf das Schu-Bild
 Data.Width    := tmpBmp.Width;                                   // setze Beite
 Data.Height   := tmpBmp.Height;                                  // setze Hhe
 if x+Data.Width+1 >= Max_X then                                  // wenn Schu-Pos auerhalb des Bildschirms ist,
    x := Max_X - Data.Width -1;                                   // korrigiere die Schuss-Position
 Data.x        := x;                                              // setze X-Position
 Data.y        := y;                                              // setze Y-Position
 Data.Speed    := C_ENEMY_SHOOT_SPEED;                            // setze Geschwindigkeit
 Add(EnemyShootLst, Data);                                        // fge Schu zur Liste hinzu
 if not PlaySound(C_SND_ENEMYFIRE, 0, SND_FILENAME or SND_ASYNC or SND_NOWAIT) then   // Gegner schiet -> Sound
    Beep;
end;


//*****************************************************************************
// Korrigiert die Bildgren, Positionen und Geschwindigkeiten der
// Gegner und der Gegner-Schsse
//
//  ->   OldField    alte Spielfeldgre
//  ->   NewField    neue Spielfeldgre
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.Resize(OldField, NewField : Integer);
var
 tmpPtr   : TFighterPtr;
begin
 SetBitmapSize(NewField);                                   // setze die Gren der Bitmaps neu
 EnSpeed := (NewField / C_DEF_PLAYFIELD) * EnSpeedOrig;     // passe Geschwindigkeit der neuen Gre an

 tmpPtr := EnemyLst;                                        // Korrigiere die Positionen der Gegner
 while tmpPtr <> nil do                                     // durchlaufe alle Gegner
   begin
   with tmpPtr^.Data.Move do
      begin
      Pos.X := (Pos.X / OldField) * NewField;               // X-Position anpassen
      Pos.Y := (Pos.Y / OldField) * NewField;               // Y-Position anpassen
      NewPos.X := (NewPos.X / OldField) * NewField;         // X-Position des Bewegungsziels anpassen
      NewPos.Y := (NewPos.Y / OldField) * NewField;         // Y-Position des Bewegungsziels anpassen
      end;
   tmpPtr := tmpPtr^.next;
   end;

 tmpPtr := EnemyShootLst;                                   // Gegner-Schsse-Positionen
 while tmpPtr <> nil do                                     // durchlaufe alle Gegner-Schsse
   begin
   with tmpPtr^.Data do
      begin
      x := PosChange(OldField, NewField, x);                // X-Position anpassen
      y := PosChange(OldField, NewField, y);                // Y-Position anpassen
      end;
   tmpPtr := tmpPtr^.next;
   end;
end;


//*****************************************************************************
// Setzt die Geschwindigkeit der Gegner neu
//
//  ->   Speed       neue Geschwindigkeit
//  ->   PlayField   Spielfeldgre
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.SetEnemySpeed(Speed : Real; PlayField : Integer);
begin
 EnSpeedOrig   := Speed;                                          // merke Original-Geschwindigkeit
 EnSpeed       := (PlayField / C_DEF_PLAYFIELD) * EnSpeedOrig;    // berechne Geschw. zur akt. Spielfeldgr.
end;


//*****************************************************************************
// ndert die Gren der Gegner-Bilder passend zur neuen Spielfeldgre
//
//  ->   NewField    neue Spielfeldgre
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.SetBitmapSize(NewField : Integer);
var
 i          : Word;
 tmpBmp     : TBitmap;
 tmpBmpOrig : TBitmap;
 Percent    : Byte;
begin
 tmpBmp     := TBitmap.Create;                              // erzeuge temp. Bild fr Grafik
 tmpBmpOrig := TBitmap.Create;                              // erzeuge temp. Bild fr Original-Grafik

 for i:=1 to EnemyBmp.Count do                              // durchlaufe jedes Bild der Liste
 begin
    tmpBmpOrig.Assign(EnemyBmpOrig.GetBmp(i, Percent) );    // ermittel Originalbild und speichere es temp.
    ResizeBmp(tmpBmp, tmpBmpOrig, NewField, Percent);       // ndere die Gre dieses temp. Bildes
    EnemyBmp.ChangeBmp(i, tmpBmp, Percent);                 // ndert das entspr. Bild in der Bilderliste
    tmpBmpOrig.FreeImage;                                   // gebe temp. Bild (nicht Objekt) wieder frei
 end;
 tmpBmp.Free;                                               // gebe temp. Bild wieder komplett frei
 tmpBmpOrig.Free;                                           // gebe temp. Bild wieder komplett frei
end;


//*****************************************************************************
// Ermittelt eine zufllige Menge an Schu-Zeitpunkten fr einen Gegner
//
//  ->   Count       Wieviele Schsse sollen pro Minute abgefeuert werden?
// <                 gibt zufll. Menge an Schu-Zeitpunkten fr Gegner zurck
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
function TEnemy.InitEnemyShoot(Count : Byte) : TEnemyShootSet;
var
 ShootSet : TEnemyShootSet;
 i        : TEnemyShoot;
begin
 ShootSet := [];                             // zunchst leere Menge
 if Count > C_MAX_ENEMY_SHOOT then           // Falls mehr Schsse angegeben als mglich
    Count := C_MAX_ENEMY_SHOOT;              // benutze Hchstwert
 while Count > 0 do                          // durchlaufe so oft, wie Schufrequenz gewhlt
 begin
    i := Random(High(TEnemyShoot) )+1;       // such Zufallswert fr Schuzeitpunkt
    if not (i in ShootSet) then              // wenn der Wert nicht schon belegt ist
    begin
       ShootSet := ShootSet + [i];           // fge Wert zur Schu-Menge hinzu
       Dec(Count);                           // und zhle den zugefgten Schu ab
    end;
 end;
 InitEnemyShoot := ShootSet;                 // gebe die Menge mit den neuen Schuzeitpunkten zurck
end;


//*****************************************************************************
// Gibt die Bilderlisten fr die Gegner (angepate- und Originalgre)
// wieder frei
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.FreeBmp;
begin
 if EnemyBmp <> nil then                     // wenn die Bilderliste existiert
 begin
    EnemyBmp.AllFree;                        // Entferne alle Bilder
    EnemyBmp.Free;                           // und gebe Objekt frei
 end;
 if EnemyBmpOrig <> nil then                 // wenn die Original-Bilderliste existiert
 begin
    EnemyBmpOrig.AllFree;                    // Entferne alle Bilder
    EnemyBmpOrig.Free;                       // und gebe Objekt frei
 end;
end;


//*****************************************************************************
// Entfernt alle Gegner und Gegner-Schsse vom Spielfeld und gibt den
// reservierten Speicher wieder frei
//
// as, 23.09.2002 - Erste Version
//*****************************************************************************
procedure TEnemy.FreeLst;
begin
 AllFree(EnemyLst);        // Entferne und gebe Gegner frei
 AllFree(EnemyShootLst);   // Entferne und gebe Gegner-Schsse frei
end;


end.
