unit UObjFire;
//*****************************************************************************
// Unit UObjFire        - Diese Unit enthlt die Klasse TFire inklusive
// ---------------        aller Deklarationen und Member-Methoden
//                      - Die Klasse TFire dient zur Anzeige von Explosionen
//                        auf dem Spielfeld
//
// ffentliche-Methoden:
// ---------------------
// Init                 - Initialsiert die Explosionsliste
// Draw                 - zeichnet die Explosionen auf der Spielflche
// AddFire              - Fgt eine neue Explosion hinzu
// FreeLst              - Gibt alle Explosionen wieder frei!
//
// Verwendet eigene Units:
// -----------------------
// UType                - Enthlt wichtige Typen
// UObjFighter          - Enthlt die Elternklasse "TFighterObject", von der
//                        sich diese Klasse ableitet
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface

uses Graphics, Controls, Types, UType, UObjFighter;

type

TFire   = class(TFighterObject)
             private
             ListPtr    : TFighterPtr;
             public
             procedure Init;
             procedure Draw(Canvas : TCanvas; ImgLst : TImageList; LookNice : Boolean);
             procedure AddFire(x,y,w,h : integer; Speed : Integer);
             procedure FreeLst;
          end;

implementation

//*****************************************************************************
// Initialisiert die Liste der Explosions-Objekte
//
// as, 05.01.2003 (14:03 Uhr) - Erste Version
//*****************************************************************************
procedure TFire.Init;
begin
 ListPtr := nil;                                                  // Liste ist leer
end;


//*****************************************************************************
// Zeichnet jede Explosion der Liste auf das Spielfeld
//
//  ->   Canvas      Zeichenflche auf der gezeichnet wird
//  ->   ImgLst      Delphi-Bilderliste mit den Bildern fr die Explosion
//  ->   LookNice    Gibt an, ob hochauflsende Explosion (true) oder
//                   einfache Expl. (false) verwendet werden
//
// as, 05.01.2003 - Erste Version
//*****************************************************************************
procedure TFire.Draw(Canvas : TCanvas; ImgLst : TImageList; LookNice : Boolean);
var
   tmpPtr   : TFighterPtr;
   delPtr   : TFighterPtr;
   tmpBmp, tmpBmp2 : TBitmap;
   dummy : boolean;
begin
 tmpPtr  := ListPtr;
 tmpBmp  := TBitmap.Create;
 tmpBmp2 := TBitmap.Create;

 while tmpPtr <> nil do                                           // durchlaufe jedes Objekt in der Liste
 begin
    delPtr := tmpPtr;
    with tmpPtr^.Data do                                          // greife auf Inhalt eines Explosions-Elements zu
    begin
    if Speed > 0 then                                             // Zeit / Geschw. der Explosion berechnen
       Dec(Speed);
    dummy := ImgLst.GetBitmap(Lives,tmpBmp);                      // hole richtiges Bild aus der Explosions-Bilder-Liste
    tmpBmp2.Width  := Width-x;                                    // passe das Bild aus der ImgLst an richtige Gre an
    tmpBmp2.Height := Height-y;

    if LookNice then                                              // sollen hochauflsende Explosionen benutzt werden?
       tmpBmp2.Canvas.StretchDraw(tmpBmp2.Canvas.ClipRect, tmpBmp)
    else
    begin
       tmpBmp2.Canvas.Brush.Color  := clYellow;
       tmpBmp2.Canvas.Pen.Width    := Lives*(Width div C_FIRE_SPEED);        // verkleinere hellen Bereich je lnger Explo. aktiv
       tmpBmp2.Canvas.Pen.Color    := clBlack;                    // -> schwarzer Rahmen wird immer grer
       tmpBmp2.Canvas.Rectangle(tmpBmp2.Canvas.ClipRect);         // zeichne Explosions-Rechteck
    end;

    Canvas.Draw(x, y, tmpBmp2);                                   // und zeichne das angepate Bild
    if Speed = 0 then                                             // wenn aktuelles Bild lange genug angezeigt
       begin
       Speed := Size;                                             // setze Wartezeit fr nchstes Bild
       Inc(Lives);                                                // und markiere nchstes Bild fr Anzeige
       end;

    tmpPtr := tmpPtr^.next;                                       // Zeiger auf nchste Explosion
    if Lives = ImgLst.Count then                                  // wenn alle Bilder der Explosion angezeigt wurden
       FreePtr(ListPtr, delPtr);                                  // entferne die Explosion aus der liste

    end;

 end;
 tmpBmp.Free;                                                     // temp. Bilder freigeben
 tmpBmp2.Free;
end;


//*****************************************************************************
// Fgt eine neue Explosion in die Liste ein
//
//  ->   x,y         Position, an der neue Explosion auf Spielfeld erscheint
//  ->   w,h         Gre (Breite und Hhe) fr neue Explosion
//  ->   Speed       Geschwindkeit fr die Explosion
//                   (wie lange sie sichtbar ist)
//
// as, 05.01.2003 - Erste Version
//*****************************************************************************
procedure TFire.AddFire(x,y,w,h : integer; Speed : Integer);
var
 Data : TFighterRec;
begin
 // flle die Objekt-Struktur
 Data.x     := x;
 Data.y     := y;
 Data.Width := w;
 Data.Height:= h;
 Data.Speed := Speed;      // aktuelle Geschwindkeit (Wert wird pro Bild runtergezhlt)
 Data.Size  := Speed;      // Kopie der Gechwindigkeit -> fr nchstes Bild init.
 Data.Lives := 0;          // welches Bild wird angezeigt?

 Add(ListPtr, Data);       // Fge das Objekt in die Liste ein
end;


//*****************************************************************************
// Entfernt alle Fire-Objekte aus der Liste und gibt den belegten Speicher
// im Heap wieder frei
//
// as, 05.01.2003 - Erste Version
//*****************************************************************************
procedure TFire.FreeLst;
begin
 AllFree(ListPtr);         // geerbte Routine gibt Liste wieder frei!
end;

end.


