unit UObjText;
//*****************************************************************************
// Unit UObjText        - Diese Unit enthlt die Klasse TText inklusive
// -------------          aller Deklarationen und Member-Methoden
//                      - Die Klasse TText dient zur Anzeige von Texten
//                        auf dem Spielfeld, die sich nach kurzer Zeit autom.
//                        ins Schwarze dimmen und wieder verschwinden
//
// ffentliche-Methoden:
// ---------------------
// Init                 - Initialsiert die Text-Objekt-Liste
// Draw                 - Zeichnet alle Text-Obj. aus der Liste und kmmert
//                        sich autom. um deren Dimmen und Verschwinden
// AddText              - Fgt zur Anzeige ein neues Text-Obj. in die Liste
// FreeLst              - Gibt die Liste mit allen Text-Obj. wieder frei
// NoText               - Gibt zurck, ob mind. ein Text-Objekt vorhanden ist
//
// Verwendet eigene Units:
// -----------------------
// UType                - Enthlt wichtige Typen
// UObjFighter          - Enthlt die Elternklasse "TFighterObject", von der
//                        sich diese Klasse ableitet
//
// Vordiploms-Programm  - (c) Januar 2003,  by Andreas Schibilla
// -------------------
//
//*****************************************************************************
interface
uses Graphics, UType, UObjFighter;

type

TText   = class(TFighterObject)
             private
             ListPtr : TFighterPtr;       // Zeiger auf den Anfang der Liste der Text-Objekte
             procedure PlaceText(var x, y : Integer; TextW, TextH, W, H : Integer; CenterW, CenterH : Boolean);

             public
             // Funktionen fr das Bearbeiten und Anzeigen der Text-Objekte
             procedure Init;
             procedure Draw(Canvas : TCanvas);
             procedure AddText(x,y : integer; Color : TColor; Size : Integer; Time : integer; Speed : Integer; Str : string);
             procedure FreeLst;
             function  NoText : Boolean;
          end;

implementation

//*****************************************************************************
// Initialisiert die Liste der Text-Objekte
//
// as, 24.09.2002 (14:03 Uhr) - Erste Version
//*****************************************************************************
procedure TText.Init;
begin
 ListPtr := nil;
end;


//*****************************************************************************
// Korrigiert die Position des Textes, so da er nicht aus dem Bild ragt
// oder zentriert ihn in horizontaler bzw. vertikaler Richtung
//
// <->   x, y        Position des Textes
//  ->   TextW       Breite des Textes in Pixel
//  ->   TextH       Hhe des Textes in Pixel
//  ->   W, H        Breite und Hhe des Spieldfeldes
//  ->   CenterW     Soll der Text horizontal zentriert werden?
//  ->   CenterH     Soll der Text vertikal zentriert werden?
//
// as, 25.09.2002 (19:42 Uhr) - Erste Version
//*****************************************************************************
procedure TText.PlaceText(var x, y : Integer; TextW, TextH, W, H : Integer; CenterW, CenterH : Boolean);
var
 Max_x    : Integer;
begin
 // korrigiere ggfs. Rand, wenn Text-Objekt zu weit rechts
 Max_x := W - TextW - 2;
 if x > Max_x then
    x := Max_x;

 if CenterW then                                   // soll Text horizontal zentriert werden?
    x := (W div 2) - (TextW div 2) - 2;            // berechne x, so dass Text zentriert ist

 if CenterH then                                   // soll Text vertikal zentriert werden?
    y := (H div 2) - (TextH div 2) - 2;            // berechne y, so dass Text zentriert ist
end;


//*****************************************************************************
// Zeichnet alle Text-Objekt aus der Liste, kmmert sich um die Warte-
// und Dimmzeit und entfernt ggfs. schwarze Objekte
//
//  ->   Canvas      Zeichenflche, auf der gezeichnet wird
//
// as, 24.09.2002 (16:51 Uhr) - Erste Version
// as, 25.09.2002 (20:14 Uhr) - Erweiterung um Text-Positions-Korrektur
//*****************************************************************************
procedure TText.Draw(Canvas : TCanvas);
var
   tmpPtr   : TFighterPtr;
   delPtr   : TFighterPtr;
begin
 tmpPtr         := ListPtr;

 while tmpPtr <> nil do                                                 // durchlaufe jedes Objekt in der Liste
 begin
    delPtr := tmpPtr;
    with tmpPtr^.Data do
    begin
       // setze Schriftattribute
       Canvas.Font.Size          := Size;
       Canvas.Font.Color         := Color;
       Canvas.Font.Name          := C_FONTNAME;
       Canvas.Brush.Style        := bsClear;

       // korrigiere die Position des Textes (Zentrieren oder Sichtbereich)
       PlaceText(x, y, Canvas.TextWidth(Str), Canvas.TextHeight(Str),   // Position und Mae des Textes
                       Canvas.ClipRect.Right, Canvas.ClipRect.Bottom,   // Mae des Spielfelds
                       Width <> 0, Height <> 0);                        // soll in x, y zentriert werden?

       Canvas.TextOut(x, y, str);                                       // schreibe Text

       if Lives = 0 then                                                // soll schon gedimmt werden? Oder noch warten?
       begin
          // Dimme und berprfe, ob Farbe schon komplett schwarz
          if Darken(Color, Speed) then
          begin
             // Entferne das schwarze Objekt
             tmpPtr := tmpPtr^.next;
             FreePtr(ListPtr, delPtr);
          end
          else
             tmpPtr := tmpPtr^.next;
       end
       else
       begin
          Dec(Lives);                                                   // Zhle Wartezeit runter und
          tmpPtr := tmpPtr^.next;                                       // gehe zum nchsten Listenelement
       end;
    end;

 end;

end;


//*****************************************************************************
// Fgt ein neues Text-Objekt in die Liste ein, dass sofort bei der nchsten
// Anzeige-Operation sichtbar wird
//
//  ->   x,y         Koordinaten, wo im Spielfeld der Text angezeigt wird
//                   (Fehler, z.B. zu weit Rechts, wird autom. korrigiert)
//                   > negative Koordinate bedeutet zentrierte Position!
//  ->   Color       Farbe der Schrift
//  ->   Size        Schriftgre
//  ->   Time        Zeit (in DrawTimer-ms), bis der Text abgedimmt wird
//                   (sobald der Text komplett schwarz ist, wird er gelscht)
//  ->   Speed       Geschwindigkeit zum Abdimmen (bis jetzt nur pos. Werte 
//                   mglich)
//  ->   Str         String, der den anzuzeigenden Text enthlt
//
// as, 24.09.2002 (16:41 Uhr) - Erste Version
//*****************************************************************************
procedure TText.AddText(x,y : integer; Color : TColor; Size : Integer; Time : Integer; Speed : Integer; Str : string);
var
 Data : TFighterRec;
begin                                                             
 // flle die Objekt-Struktur
 Data.x     := x;
 Data.y     := y;
 Data.Color := Color;
 Data.Size  := Size;
 Data.Lives := Time;
 Data.Speed := Speed;
 Data.Str   := Str;

 // Wenn Koordinate negativ -> Schrift zentrieren,
 // merke dies durch Width und Height
 if x < 0 then Data.Width  := 1 else Data.Width  := 0;
 if y < 0 then Data.Height := 1 else Data.Height := 0;

 Add(ListPtr, Data);                   // Fge das Objekt in die Liste ein
end;


//*****************************************************************************
// Entfernt alle Text-Objekte aus der Liste und gibt den belegten Speicher
// im Heap wieder frei
//
// as, 24.09.2002 (16:40 Uhr) - Erste Version
//*****************************************************************************
procedure TText.FreeLst;
begin
 AllFree(ListPtr);                     // geerbte Routine gibt Liste frei
end;


//*****************************************************************************
// berprft, ob ein Text-Objekt in der Liste ist
//
// <                 Gibt zurck, ob ein Element vorhanden ist
//
// as, 25.09.2002 (20:43 Uhr) - Erste Version
//*****************************************************************************
function TText.NoText : Boolean;
begin
 NoText := ListPtr = nil;              // Ist die Liste leer?
end;


end.
